# 🛕 ghat

> Reuse GitHub Actions workflows across repositories

You can run `ghat` **once** to set up your workflow in a new repo or run it every time you want to update it.

1. Write workflows once, use them on any repository
2. Update your workflows without copy-pasting YAML code
3. Preserve [local `env` variables](#env-object)
4. Customize workflows by [excluding jobs](#exclude-properties)

_Note: `ghat` is a tool that runs on your computer, not on GitHub Actions, and requires you to commit the changes,_ so you can be sure that the workflow never changes without you agreeing to it.

## Usage

`ghat` uses [degit](https://github.com/Rich-Harris/degit#basics) to fetch any repository or specific file/folder within it. Below you can find some examples using the workflows in [fregante/ghatemplates](https://github.com/fregante/ghatemplates).

```sh
$ ghat --help

  Usage
    $ ghat <source>

  Examples
    $ ghat fregante/ghatemplates/node
    $ ghat fregante/ghatemplates/node --exclude jobs.Build --exclude jobs.Test
    $ ghat fregante/ghatemplates/node/ci.yml

  Options:
    --exclude <dot.notation.path>  Any part of the YAML file to be removed (can be repeated)
```

### Fetch repo

If you provide a user/repo address, `ghat` will fetch the repository and look for `*.yml`/`*.yaml` files at the top level. If none are found, it will assume you want to copy the repo’s active workflows from `.github/workflows`

```sh
npx ghat fregante/ghat
# Copies *.yml OR .github/workflows/*.yml
```

### Fetch whole folder

```sh
npx ghat fregante/ghatemplates/node
# Copies node/*.yml into the local .github/workflows. It's NOT recursive
```

### Fetch specific file

```sh
npx ghat fregante/ghatemplates/node/ci.yml
# Copies node/ci.yml into the local .github/workflows/ci.yml
```

## Customization

### Exclude properties

You can exclude any property from the template by using the `--exclude` flag multiple times. This will [delete](https://github.com/sindresorhus/dot-prop) each of the specified values dot notations.

### `env` object

When you fetch a workflow that already exists locally, the local file will be overridden, except for the top-level `env` object. For example:

#### Local file

```yml
env:
  ADJECTIVE: cool

# Generated by `npx ghat fregante/workflows/demo`
# DO NOT EDIT BELOW

jobs:
  test:
    runs-on: ubuntu-latest
    steps:
      - run: echo My workflow is $ADJECTIVE
```

#### Template file

```yml
env:
  ADJECTIVE: the default

jobs:
  test:
    runs-on: ubuntu-latest
    steps:
      - run: echo This new workflow is "$ADJECTIVE" since it was updated
```

#### Updated local file

Only the top-level `env` will be preserved, the rest will be updated.

```yml
env:
  ADJECTIVE: cool

# Generated by `npx ghat fregante/workflows/demo`
# DO NOT EDIT BELOW

jobs:
  test:
    runs-on: ubuntu-latest
    steps:
      - run: echo This new workflow is "$ADJECTIVE" since it was updated
```

## What’s ghat?

**G**it**H**ub
**A**ctions
**T**emplating

I won’t pretend to know exactly what a ***Ghat*** is, but you should know [check them out,](https://en.wikipedia.org/wiki/Ghat) [they’re beautiful. 🇮🇳](https://www.gettyimages.com/global-location?requested_location=USA&requested_language=en-US&destination_url=%2Ffotos%2Fghat)


## License

MIT © [Federico Brigante](https://fregante.com)
